@@ -1,5 +1,5 @@
 /*
- * Copyright 2008, 2009 University Corporation for Atmospheric Research
+ * Copyright 2011 University Corporation for Atmospheric Research
  *
  * This file is part of the UDUNITS-2 package.  See the file LICENSE
  * in the top-level source-directory of the package for copying and
@@ -37,6 +37,9 @@ static ut_unit*		micron;
 static ut_unit*		rankine;
 static ut_unit*		celsius;
 static ut_unit*		fahrenheit;
+static ut_unit*		meterPerSecondSquared;
+static ut_unit*		meterSquaredPerSecondSquared;
+static ut_unit*		joulePerKilogram;
 static ut_unit*		watt;
 static ut_unit*		wattSquared;
 static ut_unit*		cubicMeter;
@@ -95,10 +98,12 @@ test_unitSystem(void)
     unit = ut_new_base_unit(system);
     CU_ASSERT_PTR_NOT_NULL(unit);
     CU_ASSERT_EQUAL(ut_map_unit_to_name(unit, "name", UT_ASCII), UT_SUCCESS);
+    ut_free(unit);
     unit = ut_get_dimensionless_unit_one(system);
     CU_ASSERT_PTR_NOT_NULL_FATAL(unit);
     CU_ASSERT_EQUAL(ut_format(unit, buf, sizeof(buf)-1, asciiSymbolDef), 1);
     CU_ASSERT_STRING_EQUAL(buf, "1");
+    ut_free(unit);
     ut_free_system(system);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
 }
@@ -181,6 +186,7 @@ test_utGetUnitByName(void)
 
     CU_ASSERT_PTR_NOT_NULL(altMeter);
     CU_ASSERT_EQUAL(ut_compare(altMeter, meter), 0);
+    ut_free(altMeter);
 
     CU_ASSERT_PTR_NULL(ut_get_unit_by_name(unitSystem, NULL));
     CU_ASSERT_EQUAL(ut_get_status(), UT_BAD_ARG);
@@ -197,6 +203,7 @@ test_utGetUnitBySymbol(void)
 
     CU_ASSERT_PTR_NOT_NULL(altMeter);
     CU_ASSERT_EQUAL(ut_compare(altMeter, meter), 0);
+    ut_free(altMeter);
 
     CU_ASSERT_PTR_NULL(ut_get_unit_by_symbol(unitSystem, NULL));
     CU_ASSERT_EQUAL(ut_get_status(), UT_BAD_ARG);
@@ -299,10 +306,14 @@ test_utToString(void)
     int		nchar = ut_format(meter, buf, sizeof(buf)-1, asciiSymbolDef);
     int		n;
     ut_unit*	unit;
+    const char* string = "s @ 1-01-01 00:00:00.000000 UTC";
 
     CU_ASSERT_EQUAL(nchar, 1);
     CU_ASSERT_STRING_EQUAL(buf, "m");
 
+    n = ut_format(kilogram, buf, 1, asciiSymbolDef);
+    CU_ASSERT_EQUAL(n, 2);
+
     nchar = ut_format(meter, buf, sizeof(buf)-1, asciiName);
     CU_ASSERT_STRING_EQUAL(buf, "meter");
 
@@ -313,7 +324,10 @@ test_utToString(void)
     unit = ut_parse(unitSystem, "second since 1-01-01T00:00:00Z", UT_ASCII);
     CU_ASSERT_PTR_NOT_NULL(unit);
     CU_ASSERT(ut_format(unit, buf, sizeof(buf), asciiSymbolDef) != -1);
-    CU_ASSERT_STRING_EQUAL(buf, "s @ 1-01-01 00:00:00.000000 UTC");
+    CU_ASSERT_STRING_EQUAL(buf, string);
+
+    n = ut_format(unit, buf, 1, asciiSymbolDef);
+    CU_ASSERT_EQUAL(n, strlen(string));
     ut_free(unit);
 }
 
@@ -329,7 +343,8 @@ test_utScale(void)
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
     CU_ASSERT_PTR_NOT_NULL(kilometer);
     CU_ASSERT_EQUAL(ut_get_system(meter), ut_get_system(kilometer));
-    CU_ASSERT_NOT_EQUAL(ut_compare(meter, kilometer), 0);
+    CU_ASSERT_EQUAL(ut_compare(meter, kilometer), -1);
+    CU_ASSERT_EQUAL(ut_compare(kilometer, meter), 1);
 
     nchar = ut_format(kilometer, buf, sizeof(buf)-1, asciiSymbolDef);
     CU_ASSERT_TRUE_FATAL(nchar > 0);
@@ -347,7 +362,10 @@ test_utScale(void)
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
     CU_ASSERT_PTR_NOT_NULL(micron);
     CU_ASSERT_EQUAL(ut_get_system(meter), ut_get_system(micron));
-    CU_ASSERT_NOT_EQUAL(ut_compare(meter, micron), 0);
+    CU_ASSERT_EQUAL(ut_compare(meter, micron), -1);
+    CU_ASSERT_EQUAL(ut_compare(micron, meter), 1);
+    CU_ASSERT_EQUAL(ut_compare(kilometer, micron), 1);
+    CU_ASSERT_EQUAL(ut_compare(micron, kilometer), -1);
 
     metre = ut_scale(1, meter);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
@@ -400,13 +418,17 @@ test_utOffset(void)
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
     CU_ASSERT_PTR_NOT_NULL(celsius);
     CU_ASSERT_EQUAL(ut_get_system(kelvin), ut_get_system(celsius));
-    CU_ASSERT_NOT_EQUAL(ut_compare(kelvin, celsius), 0);
+    CU_ASSERT_EQUAL(ut_compare(kelvin, celsius), -1);
+    CU_ASSERT_EQUAL(ut_compare(celsius, kelvin), 1);
 
     fahrenheit = ut_offset(rankine, 459.67);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
     CU_ASSERT_PTR_NOT_NULL(fahrenheit);
     CU_ASSERT_EQUAL(ut_get_system(rankine), ut_get_system(fahrenheit));
-    CU_ASSERT_NOT_EQUAL(ut_compare(rankine, fahrenheit), 0);
+    CU_ASSERT_EQUAL(ut_compare(rankine, fahrenheit), -1);
+    CU_ASSERT_EQUAL(ut_compare(fahrenheit, rankine), 1);
+    CU_ASSERT_EQUAL(ut_compare(celsius, fahrenheit), -1);
+    CU_ASSERT_EQUAL(ut_compare(fahrenheit, celsius), 1);
     CU_ASSERT_EQUAL(ut_map_name_to_unit("degrees_fahrenheit", UT_ASCII, fahrenheit),
 	UT_SUCCESS);
 
@@ -427,6 +449,7 @@ test_utOffset(void)
     CU_ASSERT_PTR_NOT_NULL(dupKelvin);
     CU_ASSERT_EQUAL(ut_get_system(kelvin), ut_get_system(dupKelvin));
     CU_ASSERT_EQUAL(ut_compare(kelvin, dupKelvin), 0);
+    ut_free(dupKelvin);
 
     dupKelvin = ut_offset(celsius, -273.15);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
@@ -582,6 +605,7 @@ test_utInvert(void)
     ut_unit*	inverseCelsius;
     ut_unit*	inverseRadian;
     ut_unit*	inverseMeterSecond;
+    ut_unit*	unit;
     char	buf[80];
     int		nchar;
 
@@ -621,7 +645,9 @@ test_utInvert(void)
     buf[nchar] = 0;
     CU_ASSERT_STRING_EQUAL(buf, "K-1");
 
-    inverseMeterSecond = ut_invert(ut_multiply(meter, second));
+    unit = ut_multiply(meter, second);
+    inverseMeterSecond = ut_invert(unit);
+    ut_free(unit);
     CU_ASSERT_PTR_NOT_NULL(inverseMeterSecond);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
     nchar = ut_format(inverseMeterSecond, buf, sizeof(buf)-1, asciiSymbolDef);
@@ -645,7 +671,9 @@ test_utInvert(void)
     CU_ASSERT_EQUAL(ut_map_unit_to_symbol(hertz, "Hz", UT_ASCII), UT_SUCCESS);
     CU_ASSERT_EQUAL(ut_map_symbol_to_unit("Hz", UT_ASCII, hertz), UT_SUCCESS);
 
-    megahertz = ut_scale(1e6, ut_invert(second));
+    unit = ut_invert(second);
+    megahertz = ut_scale(1e6, unit);
+    ut_free(unit);
     CU_ASSERT_PTR_NOT_NULL(megahertz);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
 }
@@ -658,6 +686,9 @@ test_utDivide(void)
     ut_unit*	kilometerPerMinute;
     ut_unit*	celsiusPerMeter;
     ut_unit*	meterPerCelsius;
+    ut_unit*	unit;
+    ut_unit*	unit2;
+    ut_unit*	unit3;
     char	buf[80];
     int		nchar;
 
@@ -697,8 +728,52 @@ test_utDivide(void)
     buf[nchar] = 0;
     CU_ASSERT_STRING_EQUAL(buf, "m.K-1");
 
-    watt = ut_divide(ut_multiply(kilogram, ut_raise(ut_divide(meter, second), 2)),
-	second);
+    unit = ut_raise(second, 2);
+    meterPerSecondSquared = ut_divide(meter, unit);
+    ut_free(unit);
+    CU_ASSERT_PTR_NOT_NULL(meterPerSecondSquared);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
+    nchar = ut_format(meterPerSecondSquared, buf, sizeof(buf)-1,
+	asciiSymbolDef);
+    CU_ASSERT_TRUE_FATAL(nchar > 0);
+    CU_ASSERT_TRUE_FATAL(nchar < sizeof(buf));
+    buf[nchar] = 0;
+    CU_ASSERT_STRING_EQUAL(buf, "m.s-2");
+
+    unit = ut_divide(meter, second);
+    meterSquaredPerSecondSquared = ut_raise(unit, 2);
+    ut_free(unit);
+    CU_ASSERT_PTR_NOT_NULL(meterSquaredPerSecondSquared);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
+    nchar = ut_format(meterSquaredPerSecondSquared, buf, sizeof(buf)-1,
+	asciiSymbolDef);
+    CU_ASSERT_TRUE_FATAL(nchar > 0);
+    CU_ASSERT_TRUE_FATAL(nchar < sizeof(buf));
+    buf[nchar] = 0;
+    CU_ASSERT_STRING_EQUAL(buf, "m2.s-2");
+
+    unit = ut_divide(meter, second);
+    unit2 = ut_raise(unit, 2);
+    ut_free(unit);
+    unit3 = ut_multiply(kilogram, unit2);
+    ut_free(unit2);
+    joulePerKilogram = ut_divide(unit3, kilogram);
+    ut_free(unit3);
+    CU_ASSERT_PTR_NOT_NULL(joulePerKilogram);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
+    nchar = ut_format(joulePerKilogram, buf, sizeof(buf)-1, asciiSymbolDef);
+    CU_ASSERT_TRUE_FATAL(nchar > 0);
+    CU_ASSERT_TRUE_FATAL(nchar < sizeof(buf));
+    buf[nchar] = 0;
+    CU_ASSERT_STRING_EQUAL(buf, "m2.s-2");
+
+    unit = ut_divide(meter, second);
+    unit2 = ut_raise(unit, 2);
+    ut_free(unit);
+    unit3 = ut_multiply(kilogram, unit2);
+    ut_free(unit2);
+    watt = ut_divide(unit3, second);
+    ut_free(unit3);
     CU_ASSERT_PTR_NOT_NULL(watt);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
     nchar = ut_format(watt, buf, sizeof(buf)-1, asciiSymbolDef);
@@ -722,6 +797,7 @@ test_utRaise(void)
 {
     ut_unit*	perCubicMeter;
     ut_unit*	celsiusCubed;
+    ut_unit*	minutesPerKilometer;
     ut_unit*	kilometersSquaredPerMinuteSquared;
     ut_unit*	unit;
     char	buf[80];
@@ -754,8 +830,9 @@ test_utRaise(void)
     buf[nchar] = 0;
     CU_ASSERT_STRING_EQUAL(buf, "K3");
 
-    kilometersSquaredPerMinuteSquared =
-	ut_raise(ut_divide(minute, kilometer), -2);
+    minutesPerKilometer = ut_divide(minute, kilometer);
+    kilometersSquaredPerMinuteSquared = ut_raise(minutesPerKilometer, -2);
+    ut_free(minutesPerKilometer);
     CU_ASSERT_PTR_NOT_NULL(kilometersSquaredPerMinuteSquared);
     CU_ASSERT_EQUAL(ut_get_status(), UT_SUCCESS);
     nchar = ut_format(kilometersSquaredPerMinuteSquared, buf,
@@ -792,21 +869,38 @@ static void
 test_utRoot(void)
 {
     ut_unit*	unit;
+    ut_unit*	unit2;
 
     CU_ASSERT_PTR_NULL(ut_root(watt, -1));
     CU_ASSERT_PTR_NULL(ut_root(watt, 0));
-    CU_ASSERT_EQUAL(ut_compare(watt, ut_root(watt, 1)), 0);
+    unit = ut_root(watt, 1);
+    CU_ASSERT_EQUAL(ut_compare(watt, unit), 0);
+    ut_free(unit);
 
     unit = ut_get_dimensionless_unit_one(unitSystem);
     CU_ASSERT_EQUAL(ut_compare(unit, ut_root(unit, 1)), 0);
-    CU_ASSERT_EQUAL(ut_compare(watt, ut_root(watt, 1)), 0);
+    ut_free(unit);
+    unit = ut_root(watt, 1);
+    CU_ASSERT_EQUAL(ut_compare(watt, unit), 0);
+    ut_free(unit);
 
-    CU_ASSERT_EQUAL(ut_compare(watt, ut_root(ut_raise(watt, 2), 2)), 0);
+    unit = ut_raise(watt, 2);
+    unit2 = ut_root(unit, 2);
+    ut_free(unit);
+    CU_ASSERT_EQUAL(ut_compare(watt, unit2), 0);
+    ut_free(unit2);
 
-    CU_ASSERT_EQUAL(ut_compare(kelvin, ut_root(ut_raise(celsius, 2), 2)), 0);
+    unit = ut_raise(celsius, 2);
+    unit2 = ut_root(unit, 2);
+    ut_free(unit);
+    CU_ASSERT_EQUAL(ut_compare(kelvin, unit2), 0);
+    ut_free(unit2);
 
-    CU_ASSERT_EQUAL(ut_compare(second, ut_root(ut_raise(secondsSinceTheEpoch,
-        2), 2)), 0);
+    unit = ut_raise(secondsSinceTheEpoch, 2);
+    unit2 = ut_root(unit, 2);
+    ut_free(unit);
+    CU_ASSERT_EQUAL(ut_compare(second, unit2), 0);
+    ut_free(unit2);
 
     CU_ASSERT_PTR_NULL(ut_root(BZ, 2));
 }
@@ -815,7 +909,8 @@ test_utRoot(void)
 static void
 test_utLog(void)
 {
-    ut_unit*	bel_1_mW = ut_log(10, ut_scale(0.001, watt));
+    ut_unit*	milliwatt = ut_scale(0.001, watt);
+    ut_unit*	bel_1_mW = ut_log(10, milliwatt);
     ut_unit*	decibel_1_mW;
     ut_unit*	unit;
     char	buf[80];
@@ -838,12 +933,14 @@ test_utLog(void)
     buf[nchar] = 0;
     CU_ASSERT_STRING_EQUAL(buf, "0.1 lg(re 0.001 kg.m2.s-3)");
 
-    unit = ut_log(-10, ut_scale(0.001, watt));
+    unit = ut_log(-10, milliwatt);
     CU_ASSERT_PTR_NULL(unit);
     CU_ASSERT_EQUAL(ut_get_status(), UT_BAD_ARG);
 
     cubicMeter = ut_raise(meter, 3);
-    cubicMicron = ut_raise(ut_scale(1e-6, meter), 3);
+    unit = ut_scale(1e-6, meter);
+    cubicMicron = ut_raise(unit, 3);
+    ut_free(unit);
     CU_ASSERT_PTR_NOT_NULL(cubicMicron);
 
     BZ = ut_log(10, cubicMicron);
@@ -871,11 +968,12 @@ test_utLog(void)
 	CU_ASSERT_EQUAL(ut_compare(unit, dBZ), 0);
 
 	CU_ASSERT_PTR_NULL(ut_get_unit_by_symbol(unitSystem, "DBZ"));
+	ut_free(unit);
     }
 
+    ut_free(milliwatt);
     ut_free(bel_1_mW);
     ut_free(decibel_1_mW);
-    ut_free(unit);
 
     CU_ASSERT_PTR_NULL(ut_log(2, NULL));
     CU_ASSERT_EQUAL(ut_get_status(), UT_BAD_ARG);
@@ -1012,6 +1110,10 @@ test_utAreConvertible(void)
     CU_ASSERT_EQUAL(ut_are_convertible(secondsSinceTheEpoch,
 	secondsSinceTheEpoch), 1);
     CU_ASSERT_EQUAL(ut_are_convertible(secondsSinceTheEpoch, second), 0);
+    CU_ASSERT_EQUAL(ut_are_convertible(joulePerKilogram, 
+	meterSquaredPerSecondSquared), 1);
+    CU_ASSERT_EQUAL(ut_are_convertible(joulePerKilogram, 
+	meterPerSecondSquared), 0);
 
     unit = ut_raise(radian, 2);
     CU_ASSERT_EQUAL(ut_are_convertible(radian, unit), 1);
@@ -1470,6 +1572,33 @@ test_utOffsetByTime(void)
 }
 
 
+static void
+test_ut_decode_time(void)
+{
+    double      timeval = ut_encode_time(2001, 1, 1, 0, 0, 0);
+    int         year1, month1, day1, hour1, minute1;
+    int         year2, month2, day2, hour2, minute2;
+    double      second1, resolution1;
+    double      second2, resolution2;
+    ut_unit*    unit;
+
+    ut_decode_time(timeval, &year1, &month1, &day1, &hour1, &minute1,
+        &second1, &resolution1);
+    unit = ut_parse(unitSystem, "second since 2010-01-11T09:08:10Z", UT_ASCII);
+    CU_ASSERT_PTR_NOT_NULL(unit);
+    ut_free(unit);
+    ut_decode_time(timeval, &year2, &month2, &day2, &hour2, &minute2,
+        &second2, &resolution2);
+    CU_ASSERT_EQUAL(year1, year2);
+    CU_ASSERT_EQUAL(month1, month2);
+    CU_ASSERT_EQUAL(day1, day2);
+    CU_ASSERT_EQUAL(hour1, hour2);
+    CU_ASSERT_EQUAL(minute1, minute2);
+    CU_ASSERT_EQUAL(second1, second2);
+    CU_ASSERT_EQUAL(resolution1, resolution2);
+}
+
+
 
 static void
 test_utSetEncoding(void)
@@ -1540,6 +1669,24 @@ test_parsing(void)
     ut_unit*	unit;
     char*	spec;
 
+    spec = "34 quatloos";
+    unit = ut_parse(unitSystem, spec, UT_ASCII);
+    CU_ASSERT_PTR_NULL(unit);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_UNKNOWN);
+
+    spec = "$&/^";
+    unit = ut_parse(unitSystem, spec, UT_ASCII);
+    CU_ASSERT_PTR_NULL(unit);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_UNKNOWN);
+
+    unit = ut_parse(unitSystem, NULL, UT_ASCII);
+    CU_ASSERT_PTR_NULL(unit);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_BAD_ARG);
+
+    unit = ut_parse(NULL, "kg", UT_ASCII);
+    CU_ASSERT_PTR_NULL(unit);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_BAD_ARG);
+
     spec = "m";
     unit = ut_parse(unitSystem, spec, UT_ASCII);
     CU_ASSERT_PTR_NOT_NULL(unit);
@@ -1763,6 +1910,11 @@ test_parsing(void)
     CU_ASSERT_PTR_NOT_NULL(unit);
     CU_ASSERT_EQUAL(ut_compare(unit, hertz), 0);
     ut_free(unit);
+
+    spec = "meter @ 100 @ 10";
+    unit = ut_parse(unitSystem, spec, UT_LATIN1);
+    CU_ASSERT_PTR_NULL(unit);
+    CU_ASSERT_EQUAL(ut_get_status(), UT_SYNTAX);
 }
 
 
@@ -2006,7 +2158,6 @@ test_xml(void)
     ut_free_system(xmlSystem);
 }
 
-
 int
 main(
     const int		argc,
@@ -2030,6 +2181,7 @@ main(
 	    CU_ADD_TEST(testSuite, test_utScale);
 	    CU_ADD_TEST(testSuite, test_utOffset);
 	    CU_ADD_TEST(testSuite, test_utOffsetByTime);
+	    CU_ADD_TEST(testSuite, test_ut_decode_time);
 	    CU_ADD_TEST(testSuite, test_utMultiply);
 	    CU_ADD_TEST(testSuite, test_utInvert);
 	    CU_ADD_TEST(testSuite, test_utDivide);
